// author: Chuck Baker
// contact: febe@customsoftwareconsult.com
// Version 7.0

function febeZip(sourceDir,destDir,zipName,absolutePath,append){
//alert(sourceDir+"\n"+destDir+"\n"+zipName)
  // Zip the contents of sourceDir (including all subdirectories) into destDir as zipName
  // If absolutePath is true, include the full pathname for each entry added to zipName
  // If append is false, create a new zipfile, otherwise append to the existing zipfile
  // Initialize
	var src;
	var root = src = sourceDir;
	var zipW;
	var febeZipFinished = false;
	
	const PR_RDONLY      = 0x01
	const PR_WRONLY      = 0x02
	const PR_RDWR        = 0x04
	const PR_CREATE_FILE = 0x08
	const PR_APPEND      = 0x10
	const PR_TRUNCATE    = 0x20
	const PR_SYNC        = 0x40
	const PR_EXCL        = 0x80
	const COMPRESSION_NONE    = 0;
	const COMPRESSION_FASTEST = 1;
	const COMPRESSION_DEFAULT = 6;
	const COMPRESSION_BEST    = 9;
	
	var ZipWriter = Components.Constructor("@mozilla.org/zipwriter;1","nsIZipWriter");
	var ZipReader = Components.Constructor("@mozilla.org/libjar/zip-reader;1","nsIZipReader", "open");

	// Init routine
	var init = function (destDir,zipName){
//alert("destdir: "+destDir+"\n"+"zipName: "+zipName)
		febeZipFinished = false;	// Set to 'true' when zipping is completed
		var observerService = new febeObserver();
		observerService.notify(null,"buItem",zipName);

		var zipFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
		zipFile.initWithPath(destDir);
		if(!zipFile.exists() || !zipFile.isDirectory()){
			zipFile.create(Ci.nsIFile.DIRECTORY_TYPE, 0755);
		}//if		
		zipFile.append(zipName);
		if (zipFile.exists() && !append){zipFile.remove(true);}
		zipW = new ZipWriter();
		if(append){
			zipW.open(zipFile, PR_RDWR | PR_CREATE_FILE | PR_APPEND);
		}else{
			zipW.open(zipFile, PR_RDWR | PR_CREATE_FILE | PR_TRUNCATE);
		}
	}//init()
	
	// Define the zip routine
	var zip = function (src){
		var aFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
		if (!aFile) return false;
		aFile.initWithPath(src);
		if(!aFile.exists()){
			var tmp = "febeZip: "+febeMsg[61].replace(/%SRC%/,src);
			febeFatal("FILE NOT FOUND",tmp);
			return talse;
		}//if
		
		var entries = aFile.directoryEntries;
		while(entries.hasMoreElements()){
			var entry = entries.getNext();
			entry.QueryInterface(Ci.nsIFile);
			var src = entry.path;
			if(entry.isDirectory()){
				zip(src);	//Recurse!
			}else{
				var file = src.substring(root.length+1);;
				if(absolutePath) file = src;
				file = file.replace(/\\/g,"/");
				
				// Ignore locked files
				if(file == "parent.lock"){continue;}
				if(file == "lock"){continue;}
				//if(file == "places.sqlite-journal"){continue;}
				if(file == "places.sqlite-stmtjrnl"){continue;}
				//if(file == "places.sqlite-journal"){continue;}
				//if(file == "cookies.sqlite-journal"){continue;}
				var jrnl = file.lastIndexOf(".sqlite-journal");
				if(jrnl > 0 && (file.length - jrnl == 15)){continue;}	// Ignore everything that ends with ".sqlite-journal"
				try{
					zipW.addEntryFile(file, Ci.nsIZipWriter.COMPRESSION_BEST, entry, false);
				}catch(e){
					febeSetMsgs();
					if(e.toString().indexOf("NS_BASE_STREAM_CLOSED") != -1){continue;}
					if(e.toString().indexOf("NS_ERROR_FILE_ALREADY_EXISTS") != -1){
						zipW.removeEntry(file, false);
						zipW.addEntryFile(file, Ci.nsIZipWriter.COMPRESSION_BEST, entry, false);
						continue;
					}//if
					if(e.toString().indexOf("NS_ERROR_NOT_INITIALIZED") != -1){
						var style = "<style>color: red;font-size: 10pt;</style>";
						var tmp = style+febeMsg[276]+"\n\n";
						style = "<style>color: black;font-size: 8pt;</style>";
						tmp += style+src+"\n\n"
						tmp += febeMsg[277];
						if(febeConfirm(tmp)){continue;}
						setTimeout("febeCloseProgressWindow()",500);
					}//if

					var msg = febeMsg[53].replace(/%FILE%/,file);
					febeFatal(e,msg);
					return false;
				}//try/catch
			}//if
		}//while
	}//zip()
	
	// nsIRequestObserver methods
	var observer = {
		onStartRequest: function(request, context){},
		onStopRequest:  function(request, context, status){
			zipW.close();
			febeZipFinished = true;
		}	  
	};
	init(destDir,zipName);
	zip(src);
	zipW.processQueue(observer,null);
	return true;
}// febeZip()

function febeUnzip(sourceDir,destDir,zipName){
  // Unzip the contents of zipName (located in sourceDir) into destDir overwriting existing entries
	var zipFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
    zipFile.initWithPath(sourceDir);
	zipFile.append(zipName);
	var zipR = Cc["@mozilla.org/libjar/zip-reader;1"].createInstance(Ci.nsIZipReader);
    zipR.open(zipFile);
	var entries = zipR.findEntries("*");
    while (entries.hasMore()) {
		var entry = entries.getNext();
		var zEntry = zipR.getEntry(entry)
		var outFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
	    outFile.initWithPath(destDir);
		var subDirs = entry.split("/");
		var end = subDirs.length;
		if(zEntry.isDirectory){--end;}
		
	    for (var i = 0; i < end; ++i) {
	      if(!febeIsDrive(subDirs[i])) outFile.append(subDirs[i]);
	      if (!(outFile.exists() || febeIsDrive(subDirs[i]))) {
				try{
					outFile.create(Ci.nsIFile.DIRECTORY_TYPE, 0755);
					//throw("error 123")
				}catch(e){
					var style = "<style>color: red;font-size: 9pt;</style>";
					var tmp = style+e.toString()+"\n\n";
					var style = "<style>color: black;font-size: 9pt;</style>";
					tmp += style+"sourceDir: "+sourceDir+"\n\n";
					tmp += style+"destDir: "+destDir+"\n\n";
					tmp += style+"zipName: "+zipName+"\n\n";
					tmp += style+"outFile: "+outFile.path+"\n\n";
					tmp += style+"length: "+outFile.path.length+"\n\n";
					
					febeAlert(tmp);
					return false;
				}
			}
	    }

	    if(!zEntry.isDirectory){
	      outFile.append(subDirs[end]);
	      zipR.extract(entry,outFile);
	    }
	}//while
	return true;
  }//febeUnzip()
  
function febeIsDrive(path){
    // Is path a drive disignator?  ("C:", etc)
	if(path.match(":")) return true
	return false;
}//febeIsDrive()
  
  function febeAddToZip(sourceNames,destDir,zipName){
  // Zip the files listed in sourceNames array into destDir as zipName -  sourceNames contains full pathnames of files only.  Directories are ignored, zipName is overwriten
	// Initialize
	var zipW;
	
	const PR_RDONLY      = 0x01
	const PR_WRONLY      = 0x02
	const PR_RDWR        = 0x04
	const PR_CREATE_FILE = 0x08
	const PR_APPEND      = 0x10
	const PR_TRUNCATE    = 0x20
	const PR_SYNC        = 0x40
	const PR_EXCL        = 0x80
	const COMPRESSION_NONE    = 0;
	const COMPRESSION_FASTEST = 1;
	const COMPRESSION_DEFAULT = 6;
	const COMPRESSION_BEST    = 9;
	
	var ZipWriter = Components.Constructor("@mozilla.org/zipwriter;1",
                                       "nsIZipWriter");
	var ZipReader = Components.Constructor("@mozilla.org/libjar/zip-reader;1",
                                       "nsIZipReader", "open");

	// Init routine
	var init = function (destDir,zipName){
		febeZipFinished = false;	// Set to 'true' when zipping is completed
		var zipFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
		zipFile.initWithPath(destDir);
		if(!zipFile.exists() || !zipFile.isDirectory()){
			zipFile.create(Ci.nsIFile.DIRECTORY_TYPE, 0755);
		}//if		
		zipFile.append(zipName);
		if (zipFile.exists()){zipFile.remove(true);}
		zipW = new ZipWriter();
		zipW.open(zipFile, PR_RDWR | PR_CREATE_FILE | PR_TRUNCATE);
	}//init()
	
	// Define the zip routine
	var zip = function (){
		for(var i in sourceNames){
			var aFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
			if (!aFile) return false;
			var entry = sourceNames[i];
			aFile.initWithPath(entry.path);
			aFile.append(entry.name);
			if(aFile.isDirectory()){
				continue;
			}else{
				try{
					zipW.addEntryFile(entry.name, Ci.nsIZipWriter.COMPRESSION_BEST, aFile, false);
				}catch(e){
					var msg = febeMsg[53].replace(/%FILE%/,entry.name)
					febeFatal(e,msg)
				}

			}//if
		}//for
	}//zip()
	
	// nsIRequestObserver methods
	var observer = {
		onStartRequest: function(request, context){},
		onStopRequest:  function(request, context, status){
			zipW.close();
			febeZipFinished = true;
		}	  
	};
	init(destDir,zipName);
	zip();
	zipW.processQueue(observer,null);
	return true;
}// febeAddToZip()